import java.awt.*;
import java.awt.event.*;
import java.io.*;

/**
 * BattleShips is designed to simulate the popular game, BattleShips
 * where each player has shots at set locations and attempts to destroy all their enemies ships
 * A ship is destroyed when all the squares it occupies have been hit
 * The Game is won when all of one players ships are destroyed
 * You can start a new game by clicking the new Game button
 * @author James Brunskill 0125699
 * @version 1.0
 */


public class BattleShips extends Frame {

    public static void main (String [ ] args)  {
	screen = new BattleShips();
	screen.show();
    }


    /**
     * This int is the initial width of the frame (can not be adjusted at run time)
     */

    public static final int FrameWidth = 660;
    
    /**
     * This int is the initial Height of the frame (can not be adjusted at run time)
     */

    public static final int FrameHeight = 400;
  

    /**
     * this int holds the size of each square on the board
     * If you were to change this you would also have to change other variables such as the framesize
     */

    public static final int squareSize = 30;


    /**
     * this int holds the number of ships each player has
     * If you were to change this you would also have to change other things such as
     * the fillShipArray() function
     */

    public static final int NUMSHIPS = 5;


    /**
     * the main variable which references the whole program
     */

    public static BattleShips screen;
    
    /**
     * This Variable holds the insets of the screen and allows us to draw things in the same place
     * Even if different platforms use more of the screen for menu bars, titles and others
     */
    private Insets ins;

    /**
     *This Dialog is used to display the Winner of the game, and exit the program
     */
    private Dialog wonDlg = new Dialog(this,"And the winner is...");
    
    /**
     *This Dialog is used when the user trys to quit, it give a list of options
     *Quit, Save and Quit, or doing nothing
     */   
    private Dialog quitDlg = new Dialog(this,"Are you sure you want to quit?");

    /**
     *This Dialog is used when the user tries to save the current game
     *it makes sure that they have set a filename and do actually want to save, not made a mistake
     */
    private Dialog saveDlg = new Dialog(this,"Saving...");

 
    /**
     *This Dialog is used when the user tries to load a saved game 
     *it makes sure that they have set a filename and do actually want to load it, not made a mistake
     */
    private Dialog loadDlg = new Dialog(this,"Loading");

    /**
     *This Dialog is used when the user wants to change the filename that you save and load from
     */
    private Dialog getFileNameDlg = new Dialog(this,"Please enter file name");
    
    /**
     *This Dialog is used when the user clicks the help button
     *It explains how to play the game
     */
    private Dialog helpDlg = new Dialog(this,"Help");


    /**
     *This Board hold all the data about the human players board
     *where the ships are, which squares have been hit etc.
     */
    public static Board youBoard;

    /**
     *This Board hold all the data about the computers board
     *where the ships are, which squares have been hit etc.
     */
    public static Board compBoard;

    /**
     * This array hold all the ships that the human player owns
     * So therefore tells where to draw them and so forth
     */
    public static Ship[] youShips = new Ship[NUMSHIPS];
 
    /**
     * This array hold all the ships that the computer player owns
     */
    public static Ship[] compShips = new Ship[NUMSHIPS];
  
    
     
    /**
     * This Panel is used to place the labels to designate the computer and the users boards
     */
    private static Panel p = new Panel();
 
    /**
     * This panel is added to the quit dialog to give the user some options
     */
    private static Panel quitp = new Panel();
 
    /**
     * This Panel is used for the save dialog
     */
    private static Panel savep = new Panel();
     
    /**
     * This Panel is used for the load dialog
     */
    private static Panel loadp = new Panel();
     
    /**
     * This panel holds all the buttons at the bottom of the main screen
     */
    private static Panel qlsp = new Panel();
 
    /**
     * This panel holds all the buttons at the bottom of winner screen
     */
    private static Panel wonp = new Panel();

 
 
    /**
     * This Panel is used for the getFilename dialog
     * it is used to add the button to hide the dialog and set the fileName String
     */
    private static Panel getFileNamep = new Panel();
    
 
    /**
     * youLabel is used to show the user where his board is, it holds the string "You"
     */
    private static Label youLabel = new Label("You");
 
    /**
     * compLabel performs a similar function to you Label by showing the user which board is the computers
     */
    private static Label compLabel = new Label("Computer");
     
    /**
     * spaceLabel is used to create space between the other labels, and position them in the right place
     */
    private static Label spaceLabel = new Label("     ");
 
    /**
     * This Label is added to the winner Dialog Box and is used to tell who the winner was
     */
    protected static Label winnerLabel = new Label("You are the Winner!",Label.CENTER);
 
    /**
     * This Label is used to ask the user if he/she is sure they want to quit
     */
    protected static Label quitingLabel = new Label("Are you sure you want to quit?",Label.CENTER);
    /**
     * This Label is used to ask the user if he/she is sure they want to save the game
     */
    protected static Label saveLabel = new Label("Are you sure you want to save?",Label.CENTER);
    /**
     * This Label is used to ask the user if he/she is sure they want to load the game
     */
    protected static Label loadLabel = new Label("Are you sure you want to load A new game?",Label.CENTER);
     /**
     * This Label prompts the user to type in the new file name
     */
    protected static Label getFileNameLabel = new Label("Please enter the file name and press Continue",Label.CENTER);

    /**
     * This is the String that the save and load functions use to reference the file
     */
    protected String fileName = "savedgame.objs";

      /**
     * This String is used to tell the user how to play and is displayed in the help dialog
     */
    protected String helpTxt = "How to Play \n When You start the game you should see two boards.\n On the left you can see some ships, these are your ships (they have been placed for you)\n to begin the game click on a square on the board marked \"computer\" the square will light up.\n If the square turns Blue then you missed, but if it turns red you scored a hit.\n Every time you click on a square on the computers board the computer trys to hit your ships on your board.\n You can watch its progress by the squares lighting up in the same manner.\n The winner is the person who gets a hit on all of the other's ships first.\n If you don't like to pattern your ships are in when you start the game, just click new game untill you are happy\n To SAVE:\n click the save button, this will present you with some options. \n You can just click save and go back to play, this will save it\n in the file savedgame.objs or the last filename you specified. \n Othewise you can click set filename and change it to store your game in the new file. \n\n To LOAD: Loading is down in a similar way, you type in the file name you want to load then click load.";

    /**
     * This TextField is used in the getfileNameInput Dialog to get an new string to place in the String fileName
     */
    protected TextField fileNameInput = new TextField("",20);
    /**
     * This TextArea is used to display the helpTxt in the Help dialog
     */
    protected TextArea helpArea = new TextArea(helpTxt,5,20);
    

    /**
     * Used to decide whether game has been started or if we are still trying to setup the board
     */
    private boolean started;


    /**
     * Used to decide whether game has been won
     */
    private boolean won;


  /**
     * The Main Constructor for the program, sets up all the panels, and dialogs, and assigns locations for the ships.
     * Adds mouselistener
     * Sets to non resizable
     * Sets Background Color
     */
    public BattleShips() {
	setTitle ("BattleShips");
	setSize (FrameWidth, FrameHeight);
	setBackground(Color.blue);
	setResizable(false);
	//setup panel...
	p.setLayout(new FlowLayout(FlowLayout.CENTER,100,2));
	p.add(youLabel);
	p.add(spaceLabel);
	p.add(spaceLabel);
	p.add(compLabel);
	add("North",p);
qlsp.add(new ButtonAdapter("New Game"){public void pressed(){newGame();}});
	qlsp.add(new ButtonAdapter("Save"){public void pressed(){saveDlg.show();}});
	qlsp.add(new ButtonAdapter("Load"){public void pressed(){loadDlg.show();}});
	qlsp.add(new ButtonAdapter("Quit"){public void pressed(){quitDlg.show();}});
	qlsp.add(new ButtonAdapter("Help"){public void pressed(){helpArea.setText(helpTxt);helpDlg.show();}});
	add("South",qlsp);
	
	//setup won dialog
	wonDlg.add("Center",winnerLabel);
	wonp.add(new ButtonAdapter("Quit"){public void pressed(){System.exit(0);}});
	wonp.add(new ButtonAdapter("Start New Game"){public void pressed(){newGame();wonDlg.hide();}});
	wonDlg.add("South",wonp);		 
	wonDlg.setVisible(false);
	wonDlg.setSize(300,100);
	wonDlg.setFont(new Font("arial",Font.PLAIN,25));
	

	//setup quit dialog
	quitDlg.add("Center",quitingLabel);
	quitp.add(new ButtonAdapter("Yes, Save and Quit"){public void pressed(){save();System.exit(0);}});
	quitp.add(new ButtonAdapter("Yes, just Quit"){public void pressed(){System.exit(0);}});
	quitp.add(new ButtonAdapter("No"){public void pressed(){quitDlg.hide();}});
	quitp.add(new ButtonAdapter("Set File Name"){public void pressed(){fileNameInput.setText(fileName);
	getFileNameDlg.show();}});
	quitDlg.add("South",quitp);
	quitDlg.setVisible(false);
	quitDlg.setSize(440,200);
	quitDlg.setFont(new Font("arial",Font.PLAIN,18));

	
	//setup save dialog
	saveDlg.add("Center",saveLabel);
	savep.add(new ButtonAdapter("Set File Name"){public void pressed(){fileNameInput.setText(fileName);
	getFileNameDlg.show();}});
	savep.add(new ButtonAdapter("Save"){public void pressed(){saveDlg.hide();save();}});
	savep.add(new ButtonAdapter("Don't Save"){public void pressed(){saveDlg.hide();}});
	saveDlg.add("South",savep);
	saveDlg.setVisible(false);
	saveDlg.setSize(440,200);
	saveDlg.setFont(new Font("arial",Font.PLAIN,18));


	//setup load dialog
	loadDlg.add("Center",loadLabel);
	loadp.add(new ButtonAdapter("Set File Name"){public void pressed(){fileNameInput.setText(fileName);
	getFileNameDlg.show();}});
	loadp.add(new ButtonAdapter("Do Nothing"){public void pressed(){loadDlg.hide();}});
	loadp.add(new ButtonAdapter("Load Old Game"){public void pressed(){loadDlg.hide();load();}});
	loadDlg.add("South",loadp);
	loadDlg.setVisible(false);
	loadDlg.setSize(580,200);
	loadDlg.setFont(new Font("arial",Font.PLAIN,18));


	//setup getFileName dialog
	getFileNameDlg.add("North",getFileNameLabel);
	getFileNameDlg.add("Center",fileNameInput);
	getFileNamep.add(new ButtonAdapter("Continue"){public void pressed(){fileName = fileNameInput.getText();getFileNameDlg.hide();}});
	getFileNameDlg.add("South",getFileNamep);
	getFileNameDlg.setVisible(false);
	getFileNameDlg.setSize(580,200);
	getFileNameDlg.setFont(new Font("arial",Font.PLAIN,18));
	fileNameInput.setBackground(Color.blue);



	//setup help dialog
	helpDlg.add("Center",helpArea);
	helpDlg.add("South",new ButtonAdapter("Quit Help"){public void pressed(){helpDlg.hide();}});
	helpDlg.setVisible(false);
	helpDlg.setSize(935,550);
	helpDlg.setFont(new Font("arial",Font.PLAIN,18));
	helpArea.setEditable(false);

	// add listeners
	addMouseListener (new MouseKeeper());
	addWindowListener (new WindowKeeper());

	//After seting up all the boarder stuff
	ins = getInsets();

	newGame();
    }


    /**
     * newGame() sets up the boards and and assigns the ships to random locations
     * Sets Started = true;
     */
    public void newGame()
    {

	youBoard = new Board(20+ins.left,50+ins.top,squareSize);
	compBoard = new Board(40+ins.left+10*30,50+ins.top,squareSize);
    
	FillShipArray(youShips);
	FillShipArray(compShips);
	
	PlaceCompShips();
	PlaceOtherShips();
	started = true;
	
	repaint();

    }

    /**
     * paint() draws the boards and the ships
     */
  public void paint (Graphics g) {
      // see the board
      youBoard.paint(g);
      compBoard.paint(g);
      
      for(int i = 0; i < NUMSHIPS;i++)
	  youShips[i].paint(g);
 

   }




    /**
     * MouseKeeper is the mouse listener used to click on the board and fire shots
     * It is also used to decided who has won etc.
     */

    private class MouseKeeper extends MouseAdapter {

	
	public void mousePressed (MouseEvent e) { 
	    int x = e.getX();
	    int y = e.getY();

	    // System.out.println("Mouse Clicked! X was " + x + " Y was " + y);
	    
	    if(started)
		{
		    if(!youBoard.won())
			{
			    if(compBoard.OnBoard(x,y))
				{
				    //System.out.println("Yes it was on the board");
				    
				    if(!compBoard.hitLoc(x,y))
					{
					    compBoard.hit(x,y);
					    if(!compBoard.won())
						{
						compMove();
						if(youBoard.won())
						    {
							started = false;
							winnerLabel.setText("The computer won");
							wonDlg.show();
						    }				

						}
					    else{
						started = false;
						winnerLabel.setText("You are the Winner !!!!");
						wonDlg.show();
					    }
					}
				}
			}
		    else
			{
			    started = false;
			    winnerLabel.setText("The computer won");
			    wonDlg.show();
					       
			    
			}
		repaint(1);
		}

	    
	    

	}
	
    }
    
    /**
     * This Function implements this programs ability to to load a previously saved gamr
     * It loads the state from the file in the String fileName
     * It does not check that the file is in the correct format so If you give it a false fileName
     * You will recieve an error You will also recieve an error if the file doesn't exist
     */

    
    public void load()
    {
	//System.out.println("Loading");
	//Setup dialog for the file input
		
		try {
		    FileInputStream f = new FileInputStream(fileName);
		    ObjectInputStream s = new ObjectInputStream(f);
		    		    
		    youBoard = (Board) s.readObject();
		    compBoard = (Board) s.readObject();
		    
		    youShips[0] = (Carrier) s.readObject();
		    youShips[1] = (Crusier) s.readObject();
		    youShips[2] = (Sub) s.readObject();
		    youShips[3] = (Destroyer) s.readObject();
		    youShips[4] = (GunBoat) s.readObject();
		    
		    compShips[0] = (Carrier) s.readObject();
		    compShips[1] = (Crusier) s.readObject();
		    compShips[2] = (Sub) s.readObject();
		    compShips[3] = (Destroyer) s.readObject();
		    compShips[4] = (GunBoat) s.readObject();
		    
		    repaint(1);
		    
		} catch (IOException e) {
		    System.out.println("received IO exception" + e);
		} catch (ClassNotFoundException e) {
		    System.out.println("received class exception" + e);
		}
	    
		//System.out.println("Loaded");
    }
    
    /**
     * This Function implements this programs ability to save the current state
     * It saves the state into the file that is refered to by the String fileName
     * It will overwrite anything previously in the file
     */

    public void save()
    {
	//System.out.println("Saving");
	//Setup dialog for the file input
		
		try  {
		    FileOutputStream f = new FileOutputStream(fileName);
		    ObjectOutputStream s = new ObjectOutputStream(f);
		    	    
		    s.writeObject(youBoard);
		    s.writeObject(compBoard);
		    
		    for(int i = 0;i < NUMSHIPS;i++)
			{
			    s.writeObject(youShips[i]);
			}
		    
		    for(int i = 0;i < NUMSHIPS;i++)
			{
			    s.writeObject(compShips[i]);
			}
		    
	    s.flush();
		} catch (IOException e){ 
		    System.out.println("received error " + e);
		}
		//  System.out.println("Saved");
    }
    
    /**
     * compMove() 
     */
    public void compMove()
    {
	int i=0,j=0;
	boolean foundMove = false;
	
	//System.out.println("CompMove");

	while(!foundMove)
	    {
		i = (int) (youBoard.BoardSize() * Math.random());
		j = (int) (youBoard.BoardSize() * Math.random());
		foundMove = !youBoard.hasBeenHit(i,j);
	    }
	
	youBoard.hitSquare(i,j);
    }


    /**
     * This Window listener opens the quit Dialog which implements the safe quit facility
     * this works in the same way as the quit button in the qls (Quit Load Save) panel
     */
    private class WindowKeeper extends WindowAdapter{
	public void  windowClosing(WindowEvent e) {
	    
	    quitDlg.show();
	}
	
    }
    /**
     * This Function Places the Computers Ships Randomly
     */

    public void PlaceCompShips()
    {
	int i = 0, j = 0;
	
	boolean ns = false;
	
	for(int k = 0; k < NUMSHIPS; k++)
	    {
		boolean placed = false;
		
		while(!placed)
		    {
			if(10*Math.random() <= 5)
			    ns = true;
			else
			    ns = false;
			
			compShips[k].setNS(ns);
			
			if(ns)
			    {
				i = (int) ((compBoard.BoardSize()) * Math.random());
				j = (int) ((compBoard.BoardSize()-compShips[k].length) * Math.random());
			    }
			else
			    {
				i = (int) ((compBoard.BoardSize()-compShips[k].length) * Math.random());
				j = (int) ((compBoard.BoardSize()) * Math.random());
			    }
			

			placed = compBoard.AddShip(i,j,compShips[k]);


		    }

		//System.out.println("Added a " + compShips[k] + " at x=" + i + " y=" +j + "CompBoard");	      
		
	    }
	
  }
    /**
     *This Function places the Users ships randomly
     */
    public void PlaceOtherShips()
    {
	//currently this places users ships but that is just temporary
		int i = 0, j = 0;
	
	boolean ns = false;
	
	for(int k = 0; k < NUMSHIPS; k++)
	    {
		
		boolean placed = false;
		
		while(!placed)
		    {
			
			if(10*Math.random() <= 5)
			    ns = true;
			else
			    ns = false;
			
			youShips[k].setNS(ns);
			
			if(ns)
			    {
				i = (int) ((youBoard.BoardSize()) * Math.random());
				j = (int) ((youBoard.BoardSize()-youShips[k].length) * Math.random());
			    }
			else
			    {
				i = (int) ((youBoard.BoardSize()-youShips[k].length) * Math.random());
				j = (int) ((youBoard.BoardSize()) * Math.random());
			    }
			
			placed = youBoard.AddShip(i,j,youShips[k]);
		    }
		//System.out.println("Added a " + youShips[k] + " at x=" + i + " y=" +j + "YouBoard");	      
	    }

	
    }
    
    /**
     *This Function places on of each kind of ship in the Ship array
     */
    public void FillShipArray(Ship[] ships)
    {
	ships[0] = new Carrier(0,0,false);
	ships[1] = new Crusier(0,0,false);
	ships[2] = new Sub(0,0,false);
	ships[3] = new Destroyer(0,0,false);
	ships[4] = new GunBoat(0,0,false);
	      

    }




    /**
     * This abstract class is used to make buttons that also implement there own listeners
     */
abstract class ButtonAdapter extends Button implements ActionListener{

    public ButtonAdapter(String name){
	super(name);
	addActionListener(this);
    }

    public void actionPerformed(ActionEvent e){pressed();}
    public abstract void pressed();
    
}


}

























