
import java.awt.*;
import java.io.Serializable;

/**
 * The board is a 10X10 array that holds the ships and stuff
 * it draws itself in a specified position
 * Sets position and size of the squares
 * @author James Brunskill 0125699
 */

public class Board implements Serializable{
    
    /**
     * Array to hold the squares for the board
     */
    protected Square[][] squares = new Square[boardsize][boardsize];
    
    /**
     * This int holds the size of the board in number of squares in each row/collum 
     */  
    final private static int boardsize = 10;

      /**
     * The X location of the board (top left) position
     */
    private int locX;

    /**
     * The Y location of the board (top left) position
     */ 
    private int locY;
    
    /**
     * This int is the size of each square, used in computing which square is hit
     */
    private int size;    

    
      /**
     * Creates a new Board with square size s and starting at the position x,y
     */
    public Board (int x, int y, int s)
    {
	locX = x;
	locY = y;
	size = s;
			
	for(int i = 0; i < boardsize; i++)
	    for(int j = 0; j < boardsize; j++)
		{
		    squares[i][j] = new Square((int)(i*s+x),(int)(j*s+y),s);
		}
	
    }
	


    /**
     * As the name suggests it returns the boardsize
     */
    public int BoardSize()
    {
	return boardsize;

    }
    /**
     * this function takes the x and y position in pixels, such as a mouse click, and returns true if it is on the board
     */

    public boolean OnBoard(int x, int y)
    {
	if(x >= locX && x <= (locX+((boardsize)*size)))
	    {
		// this means it is within x coords

		if(y > locY && y <= (locY+((boardsize)*size)))
		    {
			// it is within y cords;
			return true;

		    }
		
		else 
		    return false;

	    }
	else
	    
	    return false;

    }
    
    /**
     * takes an array index in to the board and detmines if a ship has been found at that location
     * By looking at whether it had been hit and whether it contains a ship
     */

    public boolean foundShip(int i, int j)
    {
	return squares[i][j].hasBeenHit() && squares[i][j].containsShip();
    }
    
   /**
     * Hit takes mouse xy coordinates, determines if they are on the board and then calls the hit function on the aproriate square
     * hit() on the square will set the color on the square, and return if you hit a ship or not
     */
    public boolean hit (int x, int y)
    {
	int i = 0, j = 0;
	
	if(OnBoard(x,y))
	    {
		i = (int)(x - locX) / size;
		j = (int)(y - locY) / size;
		return squares[i][j].hit();
	    }
	
	return false;
    }


    /**
     * HitLoc returns whether the square that covers the point(x,y) hasBeenHit
     */
    public boolean hitLoc (int x, int y)
    {
	int i = 0, j = 0;
	
	if(OnBoard(x,y))
	    {
		i = (int)(x - locX) / size;
		j = (int)(y - locY) / size;
		return squares[i][j].hasBeenHit();
	    }
	
	return false;
    }


    /**
     * HitLoc returns whether the square that is at array index i,j hasBeenHit yet
     */
    public boolean hasBeenHit(int i, int j)
    {return squares[i][j].hasBeenHit();}


  /**
     * Hit takes an array index and calls the hit() function on that square
     * hit() on the square will set the color on the square, and return if you hit a ship or not
     */
    public boolean hitSquare(int i, int j)
    {
	return squares[i][j].hit();
    }

    /**
     * ContainsShip takes an array index and sets the containsShip field to true on that square
     */
    public void ContainsShip(int i, int j)
    {
	squares[i][j].setContainsShip();
	
    }
 
    /**
     * doesContainShip takes an array index and returns the containsShip boolean for that square
     */
    public boolean doesContainShip(int i, int j)
    {
	return squares[i][j].containsShip();
    }

      /**
     * Adds a ship in the array starting at the index i,j it gets the direction from the passed ship
     */
    public boolean AddShip(int i, int j, Ship ship)
    {
	//still needs to avoid collisons
	
	if(ship.NS())
	    {
		//Determine if the ship can be added here
		if(i+ship.length > boardsize)
		    return false;
		for(int k = i; k < (i+ship.length);k++)
		    {
			if(squares[k][j].containsShip())
			    return false;
		    }
	
		//ship is facing north south
		for(int k = i; k < (i+ship.length);k++)
		    {
			squares[k][j].setContainsShip();
		    }
	    }
	
	else
	    {
		//Determine if the ship can be added here
		if(j+ship.length > boardsize)
		    return false;

		
		for(int k = j; k < (j+ship.length);k++)
		    {
			if(squares[i][k].containsShip())
			    return false;
		    }
	

		//ship is faceing east west
		for(int k = j; k < (j+ship.length);k++)
		    {
			squares[i][k].setContainsShip();
		    }
		
	    }
	
	ship.setLoc(squares[i][j].x(),squares[i][j].y());
    
	return true;
    }
    
    /**
     * Returns true if all the ships on the board have been sunk
     */
    public boolean won ()
    {
	
	for(int i = 0; i < boardsize; i++)
	    for(int j = 0; j < boardsize; j++)
		{
		    if(squares[i][j].containsShip())
			{
			    if(!squares[i][j].hasBeenHit())
				return false;
			    
			}
		}
           
	return true;
    }

      /**
     * paints the squares contained in the board
     */
    public void paint (Graphics g)
    {
	
	for(int i = 0; i < boardsize; i++)
	    for(int j = 0; j < boardsize; j++)
		{
		    squares[i][j].paint(g);
		}
                  
    }
}










